@@ -1,5 +1,6 @@
 # Copyright (c) OpenMMLab. All rights reserved.
 import argparse
+import copy
 import os
 import time
 
@@ -18,6 +19,11 @@ def parse_args():
     parser = argparse.ArgumentParser(description='MMDet benchmark a model')
     parser.add_argument('config', help='test config file path')
     parser.add_argument('checkpoint', help='checkpoint file')
+    parser.add_argument(
+        '--repeat-num',
+        type=int,
+        default=1,
+        help='number of repeat times of measurement for averaging the results')
     parser.add_argument(
         '--max-iter', type=int, default=2000, help='num of max iter')
     parser.add_argument(
@@ -49,7 +55,7 @@ def parse_args():
     return args
 
 
-def measure_inferense_speed(cfg, checkpoint, max_iter, log_interval,
+def measure_inference_speed(cfg, checkpoint, max_iter, log_interval,
                             is_fuse_conv_bn):
     # set cudnn_benchmark
     if cfg.get('cudnn_benchmark', False):
@@ -66,7 +72,10 @@ def measure_inferense_speed(cfg, checkpoint, max_iter, log_interval,
     data_loader = build_dataloader(
         dataset,
         samples_per_gpu=1,
-        workers_per_gpu=cfg.data.workers_per_gpu,
+        # Because multiple processes will occupy additional CPU resources,
+        # FPS statistics will be more unstable when workers_per_gpu is not 0.
+        # It is reasonable to set workers_per_gpu to 0.
+        workers_per_gpu=0,
         dist=True,
         shuffle=False)
 
@@ -123,6 +132,40 @@ def measure_inferense_speed(cfg, checkpoint, max_iter, log_interval,
     return fps
 
 
+def repeat_measure_inference_speed(cfg,
+                                   checkpoint,
+                                   max_iter,
+                                   log_interval,
+                                   is_fuse_conv_bn,
+                                   repeat_num=1):
+    assert repeat_num >= 1
+
+    fps_list = []
+
+    for _ in range(repeat_num):
+        #
+        cp_cfg = copy.deepcopy(cfg)
+
+        fps_list.append(
+            measure_inference_speed(cp_cfg, checkpoint, max_iter, log_interval,
+                                    is_fuse_conv_bn))
+
+    if repeat_num > 1:
+        fps_list_ = [round(fps, 1) for fps in fps_list]
+        times_pre_image_list_ = [round(1000 / fps, 1) for fps in fps_list]
+        mean_fps_ = sum(fps_list_) / len(fps_list_)
+        mean_times_pre_image_ = sum(times_pre_image_list_) / len(
+            times_pre_image_list_)
+        print(
+            f'Overall fps: {fps_list_}[{mean_fps_:.1f}] img / s, '
+            f'times per image: '
+            f'{times_pre_image_list_}[{mean_times_pre_image_:.1f}] ms / img',
+            flush=True)
+        return fps_list
+
+    return fps_list[0]
+
+
 def main():
     args = parse_args()
 
@@ -135,8 +178,9 @@ def main():
     else:
         init_dist(args.launcher, **cfg.dist_params)
 
-    measure_inferense_speed(cfg, args.checkpoint, args.max_iter,
-                            args.log_interval, args.fuse_conv_bn)
+    repeat_measure_inference_speed(cfg, args.checkpoint, args.max_iter,
+                                   args.log_interval, args.fuse_conv_bn,
+                                   args.repeat_num)
 
 
 if __name__ == '__main__':
